@@ -86,7 +86,7 @@ class FileObjectTests(TestCase):
     
 class HTTPTests(TestCase):
     def test_reference_file(self):
-        "download refernce file via HTTP"
+        "download reference file via HTTP"
         filename = tempfile.mktemp()
         grabber.urlgrab(ref_http, filename)
 
@@ -98,6 +98,7 @@ class HTTPTests(TestCase):
 
     def test_post(self):
         "do an HTTP post"
+        self.skip() # disabled on server
         headers = (('Content-type', 'text/plain'),)
         ret = grabber.urlread(base_http + 'test_post.php',
                               data=short_reference_data,
